import logging
import time

import numpy as np

logger = logging.getLogger(__name__)


def filter_image(image: np.ndarray, sigma_r: float, sigma_d: float):
    result = np.zeros(image.shape)
    logger.debug("Started CPU filtering")
    start_time = time.time()
    for i in range(1, image.shape[0] - 1):
        for j in range(1, image.shape[1] - 1):
            c = 0
            s = 0
            for k in range(i - 1, i + 2):
                for l in range(j - 1, j + 2):
                    g = np.exp(-((k - i) ** 2 + (l - j) ** 2) / sigma_d ** 2)
                    r = np.exp(-((image[k, l] - image[i, j]) ** 2) / sigma_r ** 2)
                    c += g * r
                    s += g * r * image[k, l]
            result[i, j] = s / c
    result_time = time.time() - start_time
    logger.debug(f"CPU execution time: {result_time} ms")
    return result, result_time
